'*********************************************************************
'*   The DISPLAY module provides most of the windowing functionality
'*   and screen I/O for this program.  At most ten windows are supported,
'*   and windows don't save what's under them when they're opened.
'*
'*********************************************************************
'
COMMON SHARED NiteMode$
DECLARE SUB FastScroll (X1 AS INTEGER, Y1 AS INTEGER, X2 AS INTEGER, Y2 AS INTEGER, NumLines AS INTEGER, Attrib AS INTEGER)
DECLARE SUB DrawBox (X1 AS INTEGER, Y1 AS INTEGER, X2 AS INTEGER, Y2 AS INTEGER, BackColor AS INTEGER)
DECLARE FUNCTION NewWindow! (X1 AS INTEGER, Y1 AS INTEGER, X2 AS INTEGER, Y2 AS INTEGER, BackColor AS INTEGER, ForeColor AS INTEGER, Title$)
DECLARE SUB InitWindows ()
DECLARE SUB Message (Text$)
DECLARE SUB WriteCS (Text$)
TYPE UWindow
	X1   AS INTEGER
	Y1   AS INTEGER
	X2   AS INTEGER
	Y2   AS INTEGER

	BC   AS INTEGER
	FC   AS INTEGER

	CurX AS INTEGER
	CurY AS INTEGER
	Title     AS STRING * 10
END TYPE
' $INCLUDE: 'QB.BI'

DIM SHARED WinArray(1 TO 10) AS UWindow

'

'*********************************************************************
'*   This is used to back up the current location of the cursor
'*   in a given window.  It is used to handle backspaces in both
'*   the terminal and prompt windows.
'*
'*********************************************************************
'
SUB BackSpace (Win AS INTEGER)

	IF (WinArray(Win).CurX > WinArray(Win).X1 + 1) THEN
		WinArray(Win).CurX = WinArray(Win).CurX - 1
	END IF

END SUB '

'*********************************************************************
'*   This DrawBox routine draws a double line frame for a window
'*   after clearing it first.  This is NOT the place to add background
'*   saving code, as it doesn't take a window handle.
'*
'*********************************************************************
'
SUB DrawBox (X1 AS INTEGER, Y1 AS INTEGER, X2 AS INTEGER, Y2 AS INTEGER, BackColor AS INTEGER)
    
'         Clear the Window
    
	IF NiteMode$ <> "Y" THEN COLOR 7, BackColor ELSE COLOR 4, 0
	FOR I = Y1 TO Y2
		LOCATE I, X1
		FOR J = X1 TO X2
			PRINT " ";
		NEXT J
	NEXT I

'         Draw the horizontal lines

	FOR I = X1 + 1 TO X2 - 1
		LOCATE Y1, I
		PRINT CHR$(205);
		LOCATE Y2, I
		PRINT CHR$(205);
	NEXT I

'         Draw the vertical lines

	FOR I = Y1 + 1 TO Y2 - 1
		LOCATE I, X1
		PRINT CHR$(186);
		LOCATE I, X2
		PRINT CHR$(186);
	NEXT I

'         Draw the corners

	LOCATE Y1, X1
	PRINT CHR$(201);
	LOCATE Y2, X1
	PRINT CHR$(200);
	LOCATE Y1, X2
	PRINT CHR$(187);
	LOCATE Y2, X2
	PRINT CHR$(188);

END SUB '

'*********************************************************************
'*   This routine makes a high speed scroll of a window possible by
'*   directly calling the video BIOS at interrupt 10H.
'*
'*
'*********************************************************************
'
SUB FastScroll (X1 AS INTEGER, Y1 AS INTEGER, X2 AS INTEGER, Y2 AS INTEGER, NumLines AS INTEGER, Attrib AS INTEGER)

	DIM InRegs AS RegType, OutRegs AS RegType

	InRegs.ax = 6 * 256 + NumLines
	InRegs.bx = Attrib * 256
	InRegs.cx = Y1 * 256 + X1
	InRegs.dx = Y2 * 256 + X2
	INTERRUPT &H10, InRegs, OutRegs

END SUB '

'*********************************************************************
'*   This routine inits the windowing system by setting each of the
'*   windows to an unused state and clearing the screen.
'*
'*
'*********************************************************************
'
SUB InitWindows

	CLS

	FOR I = 1 TO 10
		WinArray(I).X1 = -1
	NEXT I

END SUB '

'*********************************************************************
'*   LocRel moves the current cursor position in the specified window
'*   to the given relative X and Y coordinates.
'*
'*
'*********************************************************************
'
SUB LocRel (Win AS INTEGER, Rx AS INTEGER, Ry AS INTEGER)
    
	WinArray(Win).CurX = WinArray(Win).X1 + Rx
	WinArray(Win).CurY = WinArray(Win).Y1 + Ry

END SUB '

'*********************************************************************
'*   This routine is used to process things like carriage returns.  It
'*   first fills in blanks to the end of the window, and then checks
'*   to see if a scroll in necessary.  Cursor positions are also updated.
'*
'*********************************************************************
'
SUB NewLine (Win AS INTEGER)

	COLOR WinArray(Win).FC, WinArray(Win).BC

	LOCATE WinArray(Win).CurY, WinArray(Win).CurX
	FOR I = WinArray(Win).CurX TO WinArray(Win).X2 - 1
		PRINT " ";
	NEXT I

'         Now we get to scroll the window - What fun!

	IF WinArray(Win).CurY = WinArray(Win).Y2 - 1 THEN
		CALL FastScroll(WinArray(Win).X1, WinArray(Win).Y1, WinArray(Win).X2 - 2, WinArray(Win).Y2 - 2, 1, WinArray(Win).BC * 16 + WinArray(Win).FC)
	ELSE
		WinArray(Win).CurY = WinArray(Win).CurY + 1
	END IF

	WinArray(Win).CurX = WinArray(Win).X1 + 1

END SUB '

'*********************************************************************
'*   This function returns either a window handle for the new window
'*   or a zero if all ten windows have been used.
'*   The frame is also erased and drawn and the title displayed.
'*
'*********************************************************************
'
FUNCTION NewWindow (X1 AS INTEGER, Y1 AS INTEGER, X2 AS INTEGER, Y2 AS INTEGER, BackColor AS INTEGER, ForeColor AS INTEGER, Title$)

'         Find an unused window
	FOR I = 10 TO 1 STEP -1
		IF WinArray(I).X1 = -1 THEN NW = I
	NEXT I

'         If nothing could be found, then return an error code

	IF NW = 0 THEN
		NewWindow = 0
		EXIT FUNCTION
	END IF

'         Init the data in the UWindow struct

	WinArray(NW).X1 = X1
	WinArray(NW).Y1 = Y1
	WinArray(NW).X2 = X2
	WinArray(NW).Y2 = Y2
	WinArray(NW).CurX = X1 + 1
	WinArray(NW).CurY = Y1 + 1
	IF NiteMode$ = "Y" THEN
	    ForeColor = 4
	    BackColor = 0
	END IF
	WinArray(NW).BC = BackColor
	WinArray(NW).FC = ForeColor

	IF (X2 - X1 < 17) THEN
		MaxTLen = (X2 - X1) - 6
	ELSE
		MaxTLen = 11
	END IF

	WinArray(NW).Title = LEFT$(Title$, MaxTLen)

'         Draw the Window Frame

	CALL DrawBox(X1, Y1, X2, Y2, BackColor)

'         Draw the title borders
	COLOR 7, BackColor
	IF NiteMode$ = "Y" THEN COLOR 4, 0
	LOCATE Y1, ((X2 + X1) / 2) - ((LEN(Title$) + 4) / 2)
	PRINT CHR$(181); " "; Title$; " "; CHR$(198);

'         Put the title on the Frame

	COLOR 15, BackColor
	IF NiteMode$ = "Y" THEN COLOR 12, 0
	LOCATE Y1, ((X2 + X1) / 2) - (LEN(Title$) / 2)
	PRINT Title$
	NewWindow = NW

END FUNCTION '

'*********************************************************************
'*   This is the big routine that takes char of writing a string to
'*   a window.  Things considered are: clipping the string to the length
'*   of the window, and updating the current position.  DO NOT put a
'*   carriage return in the string.  That would mess it up bad!
'*********************************************************************
'
SUB WriteChars (Win AS INTEGER, ToWrite$)

	COLOR WinArray(Win).FC, WinArray(Win).BC
    
	IF LEN(ToWrite$) + WinArray(Win).CurX >= WinArray(Win).X2 THEN
		Flag% = 1
		Temp$ = LEFT$(ToWrite$, (WinArray(Win).X2 - WinArray(Win).CurX) - 1)
		LOCATE WinArray(Win).CurY, WinArray(Win).CurX, 0
		PRINT Temp$;
		PRINT RIGHT$(ToWrite$, 1);
		WinArray(Win).CurX = WinArray(Win).X2 - 1
	ELSE
		LOCATE WinArray(Win).CurY, WinArray(Win).CurX, 0
		PRINT ToWrite$;
		WinArray(Win).CurX = WinArray(Win).CurX + LEN(ToWrite$)
	END IF

END SUB '

'*********************************************************************
'*   This is basically the same as WriteChars, except it use the
'*   bright version of the window color.
'*
'*
'*********************************************************************
'
SUB WriteCharsHL (Win AS INTEGER, ToWrite$)

	COLOR WinArray(Win).FC + 8, WinArray(Win).BC
   
	IF LEN(ToWrite$) + WinArray(Win).CurX >= WinArray(Win).X2 THEN
		Flag% = 1
		Temp$ = LEFT$(ToWrite$, (WinArray(Win).X2 - WinArray(Win).CurX) - 1)
		LOCATE WinArray(Win).CurY, WinArray(Win).CurX, 0
		PRINT Temp$;
		PRINT RIGHT$(ToWrite$, 1);
		WinArray(Win).CurX = WinArray(Win).X2 - 1
	ELSE
		LOCATE WinArray(Win).CurY, WinArray(Win).CurX, 0
		PRINT ToWrite$;
		WinArray(Win).CurX = WinArray(Win).CurX + LEN(ToWrite$)
	END IF

END SUB '

