'*********************************************************************
'*   This module controls all the direct SSPCARD funtions.  Most of the
'*   code you see here can also be found (in a simplified form) in the
'*   SSPTEST.BAS program.
'*
'*
'*********************************************************************
'
DECLARE FUNCTION FRead# ()
DECLARE SUB Delay (DelayTime!)
DECLARE SUB SetCount2 (N AS LONG)
DECLARE SUB GetFiltDelay ()
DECLARE SUB SetFiltPos (F1%)
DECLARE SUB HomeFilters ()
DECLARE SUB SetInteg (N%)
DECLARE SUB PrintFilter ()
DECLARE SUB ShowInteg ()
DECLARE SUB QBMouse (a!, b!, c!, d!)
DECLARE SUB Prompt (PromptLine$, Result$)
DECLARE SUB SSPHome ()
DECLARE SUB SSPFilter (Dir%, ICnt%)
DECLARE SUB SSPIntegration ()
DECLARE FUNCTION BaseAddress% ()
DIM SHARED FilterDelay%, Period!, Frequency#, Col%, Running%, FP%

FP% = 1   'Initialize filter position counter

FUNCTION FRead#

  OUT BaseAddress% + 6, 255            'Clear counter
  OUT BaseAddress% + 6, 255
  OUT BaseAddress%, 4                  'Strobe bit 2 of control port to
  OUT BaseAddress%, 0                  ' initiate counting
  FOR I = 1 TO FilterDelay% / 10       'Delay polling of Timer 1 status
  '  J = I + 1                         ' long enough for Timer 1 output
  NEXT I                               ' to go low
  DO                                   'Poll timer 1 status until bit 7 is 1
    OUT BaseAddress% + 7, 228          ' when counting disabled
  LOOP UNTIL (INP(BaseAddress% + 5) AND 128) = 128
  OUT BaseAddress% + 7, 232            'Poll timer 1 status bit 6
  IF (INP(BaseAddress% + 6) AND 64) <> 0 THEN
    Frequency# = 0
    EXIT FUNCTION
  END IF
  'If bit 6 is 1, then get the count
  Frequency# = INP(BaseAddress% + 6) + INP(BaseAddress% + 6) * 256#
  FRead# = 65536# - Frequency#

END FUNCTION

SUB GetFiltDelay

  Dir% = 1: ICnt% = 2
  START = TIMER
  CALL SSPFilter(Dir%, ICnt%)
  TIMEDIF = TIMER - START
  IF TIMEDIF = 0 THEN TIMEDIF = 1
  FilterDelay% = INT(FilterDelay% / TIMEDIF)

END SUB

SUB HomeFilters
 
  CALL SSPHome
  CALL SetFiltPos(1)
  FP% = 1                              'Filter position counter set to 1

END SUB

SUB InitSSP

  OUT BaseAddress% + 7, 54             'Timer 0 to generate square wave
  OUT BaseAddress% + 7, 114            'Timer 1 to operate as one shot
  OUT BaseAddress% + 7, 176            'Timer 2 to operate as event counter
  OUT BaseAddress% + 4, 2              'Write divide by 2 value to Timer 0
  OUT BaseAddress% + 4, 0              ' to generate 1 kHz. LSB then MSB
  OUT BaseAddress% + 5, 1000 MOD 256   'Write value to Timer 1 such that
  OUT BaseAddress% + 5, 1000 \ 256     ' 1 sec. pulse is generated, LSB - MSB
  OUT BaseAddress% + 1, 2              'Sets output port bit D0 to 0, D1 to 1
  FilterDelay% = 50

  CALL GetFiltDelay
  CALL GetFiltDelay
  CALL GetFiltDelay
  CALL HomeFilters

END SUB

SUB SendFilter (Which%)

  FILTERTIME = TIMER
  IF Which% < FP% THEN
    Dir% = 1: ICnt% = FP% - Which%
    CALL SSPFilter(Dir%, ICnt%)
  END IF
  IF Which% > FP% THEN
    Dir% = 0: ICnt% = Which% - FP%
    CALL SSPFilter(Dir%, ICnt%)
  END IF
  FILTERTIME = TIMER - FILTERTIME
  FP% = Which%

  CALL SetFiltPos(Which%)

END SUB

SUB SendInteg (Period!)
 
'Counter integration period set
  ACTPER% = CINT(Period! * 1000)
  OUT BaseAddress% + 5, ACTPER% MOD 256
  OUT BaseAddress% + 5, ACTPER% \ 256

  CALL SetInteg(ACTPER%)

END SUB

SUB SendRead

CALL Delay(1)

  OUT BaseAddress% + 6, 255            'Clear counter
  OUT BaseAddress% + 6, 255
  OUT BaseAddress%, 4                  'Strobe bit 2 of control port to
  OUT BaseAddress%, 0                  ' initiate counting
  FOR I = 1 TO FilterDelay% / 10       'Delay polling of Timer 1 status
  '  J = I + 1                         ' long enough for Timer 1 output
  NEXT I                               ' to go low
  DO                                   'Poll timer 1 status until bit 7 is 1
    OUT BaseAddress% + 7, 228          ' when counting disabled
  LOOP UNTIL (INP(BaseAddress% + 5) AND 128) = 128
  OUT BaseAddress% + 7, 232            'Poll timer 1 status bit 6
  IF (INP(BaseAddress% + 6) AND 64) <> 0 THEN
    Frequency# = 0
    EXIT SUB
  END IF
  'If bit 6 is 1, then get the count
  Frequency# = INP(BaseAddress% + 6) + INP(BaseAddress% + 6) * 256#
  Count# = 65536# - Frequency#
   
  TheCount& = Count#
  CALL SetCount2(TheCount&)

END SUB

SUB SSPFilter (Dir%, ICnt%)

  IF (Dir% <> 1) AND (Dir% <> 0) THEN
    EXIT SUB
  END IF
  OUT BaseAddress%, Dir% * 2           'Set direction
  FOR I = 1 TO ICnt%
    FOR J = 1 TO 33
      FOR K = 1 TO FilterDelay%        'Delay statement
      NEXT K
      OUT BaseAddress%, (Dir% * 2 + 1) 'Move motor 1 step
      OUT BaseAddress%, (Dir% * 2)
    NEXT J
  NEXT I

END SUB

SUB SSPHome

  OUT BaseAddress% + 1, 0              'Reset motor controller
  OUT BaseAddress% + 1, 2
  FOR I = 1 TO 300
    FOR K = 1 TO FilterDelay%          'Delay statement
    NEXT K
    OUT BaseAddress%, 3                'Move filter rack 300 steps in
    OUT BaseAddress%, 2                'direction 1
  NEXT I

END SUB

SUB TestPorts

  DO
    LOCATE 20, 1, 1
    PRINT "                                                            "
    LOCATE 20, 1, 1
    Prompt "Select Ports to Test - Input, Output, or eXit ", IO$
    IF UCASE$(LEFT$(IO$, 1)) = "I" THEN
      LOCATE 20, 43, 0: PRINT "INPUT D0 ="
      LOCATE 21, 43, 0: PRINT "INPUT D1 ="
      LOCATE 22, 43, 0: PRINT "INPUT D4 ="
      ADDRESS% = BaseAddress% + 2      'ADDRESS% needed to speed up INP
      DO                               'function.
        INPUTS% = INP(ADDRESS%)
        IF INPUTS% = 0 THEN
          LOCATE 20, 58, 0: PRINT "LO"
          LOCATE 21, 58, 0: PRINT "LO"
          LOCATE 22, 58, 0: PRINT "LO"
        END IF
        IF INPUTS% = 1 THEN
          LOCATE 20, 58, 0: PRINT "HI"
          LOCATE 21, 58, 0: PRINT "LO"
          LOCATE 22, 58, 0: PRINT "LO"
        END IF
        IF INPUTS% = 2 THEN
          LOCATE 20, 58, 0: PRINT "LO"
          LOCATE 21, 58, 0: PRINT "HI"
          LOCATE 22, 58, 0: PRINT "LO"
        END IF
        IF INPUTS% = 3 THEN
          LOCATE 20, 58, 0: PRINT "HI"
          LOCATE 21, 58, 0: PRINT "HI"
          LOCATE 22, 58, 0: PRINT "LO"
        END IF
        IF INPUTS% = 16 THEN
          LOCATE 20, 58, 0: PRINT "LO"
          LOCATE 21, 58, 0: PRINT "LO"
          LOCATE 22, 58, 0: PRINT "HI"
        END IF
        IF INPUTS% = 17 THEN
          LOCATE 20, 58, 0: PRINT "HI"
          LOCATE 21, 58, 0: PRINT "LO"
          LOCATE 22, 58, 0: PRINT "HI"
        END IF
        IF INPUTS% = 18 THEN
          LOCATE 20, 58, 0: PRINT "LO"
          LOCATE 21, 58, 0: PRINT "HI"
          LOCATE 22, 58, 0: PRINT "HI"
        END IF
        IF INPUTS = 19 THEN
          LOCATE 20, 58, 0: PRINT "HI"
          LOCATE 21, 58, 0: PRINT "HI"
          LOCATE 22, 58, 0: PRINT "HI"
        END IF
      LOOP UNTIL UCASE$(INKEY$) = "X"
      EXIT SUB
    END IF
    IF UCASE$(LEFT$(IO$, 1)) = "O" THEN
      DO
        LOCATE 20, 1, 1
        PRINT "                                                           "
        LOCATE 20, 1, 1: INPUT "OUTPUT D0, D1 OR D2 - ENTER 0, 1 OR 2", DX$
      LOOP UNTIL DX$ = "0" OR DX$ = "1" OR DX$ = "2"
      DSTATE% = 0
      DO
        LOCATE 20, 1, 1
        INPUT "PRESS H OR L TO TOGGLE HIGH OR LOW OUTPUT, X TO STOP ", X$
      IF X$ = "H" AND DX$ = "0" THEN
        DSTATE% = 1 OR DSTATE%
        OUT (BaseAddress% + 1), DSTATE%
        LOCATE 17, 42, 1: PRINT "OUTPUT D0 =     HI"
      END IF
      IF X$ = "L" AND DX$ = "0" THEN
        DSTATE% = 6 AND DSTATE%
        OUT (BaseAddress% + 1), DSTATE%
        LOCATE 17, 42, 1: PRINT "OUTPUT D0 =     LO"
        END IF
      IF X$ = "H" AND DX$ = "1" THEN
        DSTATE% = 2 OR DSTATE%
        OUT (BaseAddress% + 1), DSTATE%
        LOCATE 18, 42, 1: PRINT "OUTPUT D1 =     HI"
      END IF
      IF X$ = "L" AND DX$ = "1" THEN
        DSTATE% = 5 AND DSTATE%
        OUT (BaseAddress% + 1), DSTATE%
        LOCATE 18, 42, 1: PRINT "OUTPUT D1 =     LO"
        END IF
      IF X$ = "H" AND DX$ = "2" THEN
        DSTATE% = 4 OR DSTATE%
        OUT (BaseAddress% + 1), DSTATE%
        LOCATE 19, 42, 1: PRINT "OUTPUT D2 =     HI"
        END IF
      IF X$ = "L" AND DX$ = "2" THEN
        DSTATE% = 3 AND DSTATE%
        OUT (BaseAddress% + 1), DSTATE%
        LOCATE 19, 42, 1: PRINT "OUTPUT D2 =     LO"
      END IF
      LOOP UNTIL X$ = "X"
    END IF
  LOOP UNTIL LEFT$(IO$, 1) = "X"

END SUB

